#!/usr/bin/env python3
from threading import Timer
import sys, time
from daemon import daemon
from analysis_port import analysis_port_status
import pyufw as ufw
import json

def check_condition(*args): 
    port=args[0]
    established=args[1]
    syn_recv=args[2]
    close_wait=args[3]
    
    status_list=analysis_port_status(port)
    for i, status in enumerate(status_list):
         do_block=False
         if established>0 and status['established']>=established:
              do_block=True
         if syn_recv>0 and status['syn_recv']>=syn_recv:
              do_block=True
         if close_wait>0 and status['close_wait']>=close_wait:
              do_block=True
         if do_block==True:
              rule="deny from "+str(status['ip'])+" to any port "+str(port) 
              ufw.add(rule)

class RepeatingTimer(Timer): 
 def run(self):
  while not self.finished.is_set():
   self.finished.wait(self.interval)
   self.function(*self.args, **self.kwargs)

class MyDaemon(daemon):
      
      timer_list=list()

      def run(self):
          self.start_working()
          while True:
            time.sleep(1)
      
      def init(self):
          self.timer_list.clear()
          with open('/app/Firewall/blocklist/rule_config.json', 'r') as f:
            data = json.load(f)
            for i, element in enumerate(data):
             interval_time=element['interval_time']
             port=element['port']
             established=element['established']
             syn_recv=element['syn_recv']
             close_wait=element['close_wait']
             timer=RepeatingTimer(interval_time,check_condition,(port,established,syn_recv,close_wait,))
             self.timer_list.append(timer)

      def start_working(self):
          for timer in self.timer_list:
              timer.start()

      def stop_working(self):  
          for timer in self.timer_list:
              timer.cancel()

if __name__ == "__main__":
        daemon = MyDaemon('/tmp/daemon-blocklist.pid')
        if len(sys.argv) == 2:
                if 'start' == sys.argv[1]:
                        daemon.start()
                elif 'stop' == sys.argv[1]:
                        daemon.stop()
                elif 'restart' == sys.argv[1]:
                        daemon.restart()
                else:
                        print('Unknown command')
                        sys.exit(2)
                sys.exit(0)
        else:
                print( "usage: %s start|stop|restart" % sys.argv[0])
                sys.exit(2)
